from clipass.core.models.flags import FlagModel
import random

class LengthModel(FlagModel):

    def __init__(self):
        self.short_name = "l"
        self.long_name = "length"
        self.help = "Password length"
        self.default_value = 10
        super().__init__()


    def __call__(self, *args, **kwargs):
        super().__call__(*args, **kwargs)

        loops = 0
        new_password = ""
        while loops < int(kwargs["value"]):
            new_password += kwargs["password"][random.randint(0, len(kwargs["password"])-1)]
            loops += 1

        return new_password

#init your model here
LengthModel()


from ...models.flags import FlagModel


class MaxModel(FlagModel):
    __instance = None

    def __init__(self):
        if __class__.__instance is not None:
            raise ("Singleton!")

        self.short_name = "m"
        self.long_name = "max"
        self.help = "max passwords"
        self.default_value = 16
        super().__init__()

        __class__.__instance = self

    @staticmethod
    def get_instance():
        if __class__.__instance is None:
            __class__.__init__()
        return __class__.__instance

    def get_default_value(self):
        return self.default_value

    def __call__(self, *args, **kwargs):
        return super().__call__(*args, **kwargs)


MaxModel()



class PasswordGenerator():
    allowed_chars = "abcdefghiklmnoqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890!@#$%^&*()_+~:;"

    def __init__(self):
        self.generate()


    def generate(self):
        self._pass = __class__.allowed_chars
        return self.get_password

    @property
    def get_password(self):
        return self._pass

    @get_password.setter
    def set_password(self, password):
        self._pass = password
        return self


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

#packaging
setuptools.setup(
    name="clipass",
    version="0.0.1",
    author="Itay Bardugo",
    author_email="itaybardugo91@gmail.com",
    description="CLI passowrd generator",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/itay-bardugo/clipass",
    dependency_links=[],
    entry_points={
        "console_scripts": [
            "clipass = clipass.main:main"
        ],
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from .core.commands.args import received_options
from .core.models.flags import FlagModel
from .core.base_flags.max.model import MaxModel
from .core.models.flag_error import FlagError


def generate_password():
    from .core.models.password_generator import PasswordGenerator
    PasswordGenerator = PasswordGenerator()
    output = []

    try:
        while True:
            passwd = PasswordGenerator.generate()
            new_pass = None

            for option, value in received_options.__dict__.items():
                try:
                    Model = FlagModel.all.get(option, False)
                    if not isinstance(Model, FlagModel):
                        raise Exception("your model is not an instance of FlagModel")

                except Exception as e:
                    raise Exception(str(e))

                else:
                    try:
                        new_pass = Model(password=(new_pass or passwd), value=value)
                    except FlagError as e:
                        #one failed, restart.
                        new_pass = None
                        break

            if new_pass is not None:
                if int(output.__len__()) >= int(MaxModel.get_instance().get_default_value()):
                    raise Exception("finished?")
                else:
                    output.append(str(new_pass))
    except:
        loop = 1
        for generated_password in output:
            print(generated_password, end="\n" if loop % 4 == 0 else " ")
            loop += 1



from ...flags import parser

class FlagModel():
    _pass = ""
    all = {}

    def __init__(self, long_name="", short_name="", help_str="", default_value=False):
        self.long_name = getattr(self, "long_name", False) or long_name
        self.short_name = getattr(self, "short_name", False) or short_name
        self.help = getattr(self, "help", False) or help_str
        self.default_value = getattr(self, "default_value", False) or default_value
        super().__init__()

        if __class__.all.get(self.long_name, False):
            raise Exception("Flag '{}' already exists.".format(self.long_name))

        optionals_params_kwargs = {}
        if self.help:
            optionals_params_kwargs["help"] = self.help

        if self.default_value:
            optionals_params_kwargs["default"] = self.default_value

        try:
            parser.add_option(
                "-{}".format(self.short_name),
                "--{}".format(self.long_name),
                **optionals_params_kwargs
            )
        except Exception as e:
            print(str(e))
        else:
            __class__.all[self.long_name] = self

    def get_all(self):
        return self.all

    def __call__(self, *args, **kwargs):
        """return true or false according to your flag logic"""
        if kwargs["value"]:
            self.default_value = kwargs["value"]

        return kwargs.get("password", False)
